from airflow.triggers.base import BaseTrigger, TriggerEvent
from google.cloud import dataflow_v1beta3


class DataflowTrigger(BaseTrigger):
    def __init__(self, project_id, region, body):
        super().__init__()
        self.project_id = project_id
        self.region = region
        self.body = body

    def serialize(self):
        """Serialize the trigger for Airflow's triggerer."""
        return ("dataflow_trigger.DataflowTrigger", {
            "project_id": self.project_id,
            "region": self.region,
            "body": self.body
        })

    async def run(self):
        """Trigger logic to start the Dataflow job."""
        try:
            client = dataflow_v1beta3.FlexTemplatesServiceAsyncClient()
            request = dataflow_v1beta3.LaunchFlexTemplateRequest(
                project_id=self.project_id,
                location=self.region,
                launch_flex_template_parameter=self.body
            )
            response = await client.launch_flex_template(request=request)
            yield TriggerEvent({"status": "success", "jobId": response.job.id})
        except Exception as e:
            yield TriggerEvent({"status": "error", "message": str(e)})
