from airflow.models.baseoperator import BaseOperator
from airflow.utils.context import Context
from dataflow_trigger import DataflowTrigger


class DeferrableDataflowOperator(BaseOperator):
    def __init__(self, project_id, region, body, **kwargs):
        super().__init__(**kwargs)
        self.project_id = project_id
        self.region = region
        self.body = body

    def execute(self, context: Context):
        """Defers execution to the triggerer."""
        self.defer(
            trigger=DataflowTrigger(
                project_id=self.project_id,
                region=self.region,
                body=self.body,
            ),
            method_name="execute_complete",
        )

    def execute_complete(self, context: Context, event: dict):
        """Callback method executed once trigger completes."""
        if event["status"] == "success":
            self.log.info(
                f"Dataflow job started successfully! Job ID: {event['jobId']}")
        else:
            raise RuntimeError(f"Dataflow job failed: {event['message']}")
